@@ -37,17 +37,16 @@
 #include "gegl-sampler-linear.h"
 #include "gegl-sampler-cubic.h"
 #include "gegl-sampler-lanczos.h"
-#include "gegl-sampler-sharp.h"
-#include "gegl-sampler-yafr.h"
+#include "gegl-sampler-downsharp.h"
+#include "gegl-sampler-downsize.h"
+#include "gegl-sampler-downsmooth.h"
+#include "gegl-sampler-upsharp.h"
+#include "gegl-sampler-upsize.h"
+#include "gegl-sampler-upsmooth.h"
 #include "gegl-buffer-index.h"
 #include "gegl-tile-backend.h"
 #include "gegl-buffer-iterator.h"
 
-#if ENABLE_MP
-GStaticRecMutex mutex = G_STATIC_REC_MUTEX_INIT;
-#endif
-
-
 #if 0
 static inline void
 gegl_buffer_pixel_set (GeglBuffer *buffer,
@@ -107,7 +106,7 @@ gegl_buffer_pixel_set (GeglBuffer *buffer,
               memcpy (tp, buf, bpx_size);
 
             gegl_tile_unlock (tile);
-            g_object_unref (tile);
+            gegl_tile_unref (tile);
           }
       }
   }
@@ -184,7 +183,7 @@ gegl_buffer_set_pixel (GeglBuffer *buffer,
           {
             if (buffer->hot_tile)
               {
-                g_object_unref (buffer->hot_tile);
+                gegl_tile_unref (buffer->hot_tile);
                 buffer->hot_tile = NULL;
               }
             tile = gegl_tile_source_get_tile ((GeglTileSource *) (buffer),
@@ -262,7 +261,7 @@ gegl_buffer_get_pixel (GeglBuffer *buffer,
           {
             if (buffer->hot_tile)
               {
-                g_object_unref (buffer->hot_tile);
+                gegl_tile_unref (buffer->hot_tile);
                 buffer->hot_tile = NULL;
               }
             tile = gegl_tile_source_get_tile ((GeglTileSource *) (buffer),
@@ -281,7 +280,7 @@ gegl_buffer_get_pixel (GeglBuffer *buffer,
             else
               memcpy (buf, tp, px_size);
 
-            /*g_object_unref (tile);*/
+            /*gegl_tile_unref (tile);*/
             buffer->hot_tile = tile;
           }
       }
@@ -294,19 +293,22 @@ gegl_buffer_get_pixel (GeglBuffer *buffer,
 void
 gegl_buffer_flush (GeglBuffer *buffer)
 {
+  GeglTileBackend *backend;
+
   g_return_if_fail (GEGL_IS_BUFFER (buffer));
+  backend = gegl_buffer_backend (buffer);
 
   if (buffer->hot_tile)
     {
-      g_object_unref (buffer->hot_tile);
+      gegl_tile_unref (buffer->hot_tile);
       buffer->hot_tile = NULL;
     }
-  if ((GeglBufferHeader*)(gegl_buffer_backend (buffer)->header))
+  if ((GeglBufferHeader*)(backend->header))
     {
-      ((GeglBufferHeader*)(gegl_buffer_backend (buffer)->header))->x =buffer->extent.x;
-      ((GeglBufferHeader*)(gegl_buffer_backend (buffer)->header))->y =buffer->extent.y;
-      ((GeglBufferHeader*)(gegl_buffer_backend (buffer)->header))->width =buffer->extent.width;
-      ((GeglBufferHeader*)(gegl_buffer_backend (buffer)->header))->height =buffer->extent.height;
+      ((GeglBufferHeader*)(backend->header))->x =buffer->extent.x;
+      ((GeglBufferHeader*)(backend->header))->y =buffer->extent.y;
+      ((GeglBufferHeader*)(backend->header))->width =buffer->extent.width;
+      ((GeglBufferHeader*)(backend->header))->height =buffer->extent.height;
     }
 
   gegl_tile_source_command (GEGL_TILE_SOURCE (buffer),
@@ -315,7 +317,7 @@ gegl_buffer_flush (GeglBuffer *buffer)
 
 
 
-static void inline
+static inline void
 gegl_buffer_iterate (GeglBuffer          *buffer,
                      const GeglRectangle *roi, /* or NULL for extent */
                      guchar              *buf,
@@ -575,7 +577,7 @@ gegl_buffer_iterate (GeglBuffer          *buffer,
                         bp += buf_stride;
                       }
                   }
-                g_object_unref (tile);
+                gegl_tile_unref (tile);
               }
             bufx += (tile_width - offsetx);
           }
@@ -583,6 +585,33 @@ gegl_buffer_iterate (GeglBuffer          *buffer,
     }
 }
 
+void
+gegl_buffer_set_unlocked (GeglBuffer          *buffer,
+                          const GeglRectangle *rect,
+                          const Babl          *format,
+                          void                *src,
+                          gint                 rowstride)
+{
+  if (format == NULL)
+    format = buffer->format;
+
+#if 0 /* XXX: not thread safe */
+  if (rect && rect->width == 1 && rect->height == 1) /* fast path */
+    {
+      gegl_buffer_set_pixel (buffer, rect->x, rect->y, format, src);
+    }
+  else
+#endif
+    {
+      gegl_buffer_iterate (buffer, rect, src, rowstride, TRUE, format, 0);
+    }
+
+  if (gegl_buffer_is_shared(buffer))
+    {
+      gegl_buffer_flush (buffer);
+    }
+}
+
 void
 gegl_buffer_set (GeglBuffer          *buffer,
                  const GeglRectangle *rect,
@@ -592,35 +621,14 @@ gegl_buffer_set (GeglBuffer          *buffer,
 {
   g_return_if_fail (GEGL_IS_BUFFER (buffer));
 
-#if ENABLE_MP
-  g_static_rec_mutex_lock (&mutex);
-#endif
   gegl_buffer_lock (buffer);
-
-  if (format == NULL)
-    format = buffer->format;
-
-  if (rect && rect->width == 1 && rect->height == 1) /* fast path */
-    {
-      gegl_buffer_set_pixel (buffer, rect->x, rect->y, format, src);
-    }
-  else
-    {
-      gegl_buffer_iterate (buffer, rect, src, rowstride, TRUE, format, 0);
-    }
-
-  if (gegl_buffer_is_shared(buffer))
-    {
-      gegl_buffer_flush (buffer);
-    }
-  gegl_buffer_unlock (buffer); /* XXX: should this happen before flush? */
-#if ENABLE_MP
-  g_static_rec_mutex_unlock (&mutex);
-#endif
+  gegl_buffer_set_unlocked (buffer, rect, format, src, rowstride);
+  gegl_buffer_unlock (buffer);
 }
 
 
 
+
 #if 0
 
 /*
@@ -935,56 +943,44 @@ resample_boxfilter_u8 (void   *dest_buf,
     }
 }
 
+
 void
-gegl_buffer_get (GeglBuffer          *buffer,
-                 gdouble              scale,
-                 const GeglRectangle *rect,
-                 const Babl          *format,
-                 gpointer             dest_buf,
-                 gint                 rowstride)
+gegl_buffer_get_unlocked (GeglBuffer          *buffer,
+                          gdouble              scale,
+                          const GeglRectangle *rect,
+                          const Babl          *format,
+                          gpointer             dest_buf,
+                          gint                 rowstride)
 {
-  g_return_if_fail (GEGL_IS_BUFFER (buffer));
-#if ENABLE_MP
-  g_static_rec_mutex_lock (&mutex);
-#endif
 
   if (format == NULL)
     format = buffer->format;
 
+#if 0
+  /* not thread-safe */
   if (scale == 1.0 &&
       rect &&
       rect->width == 1 &&
       rect->height == 1)  /* fast path */
     {
       gegl_buffer_get_pixel (buffer, rect->x, rect->y, format, dest_buf);
-#if ENABLE_MP
-      g_static_rec_mutex_unlock (&mutex);
-#endif
       return;
     }
+#endif
 
   if (!rect && scale == 1.0)
     {
       gegl_buffer_iterate (buffer, NULL, dest_buf, rowstride, FALSE, format, 0);
-#if ENABLE_MP
-      g_static_rec_mutex_unlock (&mutex);
-#endif
       return;
     }
   if (rect->width == 0 ||
       rect->height == 0)
     {
-#if ENABLE_MP
-      g_static_rec_mutex_unlock (&mutex);
-#endif
       return;
     }
   if (GEGL_FLOAT_EQUAL (scale, 1.0))
     {
       gegl_buffer_iterate (buffer, rect, dest_buf, rowstride, FALSE, format, 0);
-#if ENABLE_MP
-      g_static_rec_mutex_unlock (&mutex);
-#endif
       return;
     }
   else
@@ -1066,9 +1062,18 @@ gegl_buffer_get (GeglBuffer          *buffer,
         }
       g_free (sample_buf);
     }
-#if ENABLE_MP
-  g_static_rec_mutex_unlock (&mutex);
-#endif
+}
+
+void
+gegl_buffer_get (GeglBuffer          *buffer,
+                 gdouble              scale,
+                 const GeglRectangle *rect,
+                 const Babl          *format,
+                 gpointer             dest_buf,
+                 gint                 rowstride)
+{
+  g_return_if_fail (GEGL_IS_BUFFER (buffer));
+  gegl_buffer_get_unlocked (buffer, scale, rect, format, dest_buf, rowstride);
 }
 
 const GeglRectangle *
@@ -1097,16 +1102,15 @@ gegl_buffer_sample (GeglBuffer       *buffer,
   return;
 #endif
 
-#if ENABLE_MP
-  g_static_rec_mutex_lock (&mutex);
-#endif
-
   desired_type = gegl_sampler_type_from_interpolation (interpolation);
 
+  /* unset the cached sampler if it dosn't match the needs */
   if (buffer->sampler != NULL &&
-      !G_TYPE_CHECK_INSTANCE_TYPE (buffer->sampler, desired_type))
+     (!G_TYPE_CHECK_INSTANCE_TYPE (buffer->sampler, desired_type) || 
+       buffer->sampler_format != format
+      ))
     {
-      g_object_unref(buffer->sampler);
+      g_object_unref (buffer->sampler);
       buffer->sampler = NULL;
     }
 
@@ -1117,20 +1121,12 @@ gegl_buffer_sample (GeglBuffer       *buffer,
                                       "buffer", buffer,
                                       "format", format,
                                       NULL);
+      buffer->sampler_format = format;
       gegl_sampler_prepare (buffer->sampler);
     }
   gegl_sampler_get (buffer->sampler, x, y, dest);
 
-#if ENABLE_MP
-  g_static_rec_mutex_unlock (&mutex);
-#endif
-
-  /* if none found, create a singleton sampler for this buffer,
-   * a function to clean up the samplers set for a buffer should
-   * also be provided */
-
   /* if (scale < 1.0) do decimation, possibly using pyramid instead */
-
 }
 
 void
@@ -1239,13 +1235,5 @@ gegl_buffer_sampler (GeglBuffer       *buffer,
   g_return_if_fail (GEGL_IS_BUFFER (buffer));
   g_return_if_fail (GEGL_IS_SAMPLER (sampler));
 
-#if ENABLE_MP
-  g_static_rec_mutex_lock (&mutex);
-#endif
-
   gegl_sampler_get (sampler, x, y, dest);
-
-#if ENABLE_MP
-  g_static_rec_mutex_unlock (&mutex);
-#endif
 }
