@@ -63,18 +63,24 @@
 #include "gegl-sampler-linear.h"
 #include "gegl-sampler-cubic.h"
 #include "gegl-sampler-lanczos.h"
-#include "gegl-sampler-sharp.h"
-#include "gegl-sampler-yafr.h"
+#include "gegl-sampler-downsharp.h"
+#include "gegl-sampler-downsize.h"
+#include "gegl-sampler-downsmooth.h"
+#include "gegl-sampler-upsharp.h"
+#include "gegl-sampler-upsize.h"
+#include "gegl-sampler-upsmooth.h"
 #include "gegl-types-internal.h"
 #include "gegl-utils.h"
 #include "gegl-id-pool.h"
 #include "gegl-buffer-index.h"
 #include "gegl-config.h"
 
+/* #define GEGL_BUFFER_DEBUG_ALLOCATIONS */
+
 /* #define GEGL_BUFFER_DEBUG_ALLOCATIONS to print allocation stack
  * traces for leaked GeglBuffers using GNU C libs backtrace_symbols()
  */
-#ifdef GEGL_BUFFER_DEBUG_ALLOCATIONS
+#ifdef HAVE_EXECINFO_H
 #include <execinfo.h>
 #endif
 
@@ -109,7 +115,7 @@ enum {
   LAST_SIGNAL
 };
 
-static GeglBuffer *gegl_buffer_new_from_format     (const void *babl_format,
+static GeglBuffer *gegl_buffer_new_from_format     (const void *babl_fmt,
                                                     gint        x,
                                                     gint        y,
                                                     gint        width,
@@ -315,7 +321,7 @@ gegl_buffer_set_extent (GeglBuffer          *buffer,
     {
       GeglBufferHeader *header = ((GeglBufferHeader*)(gegl_buffer_backend (buffer)->header));
       header->x = buffer->extent.x;
-      header->y = buffer->extent.x;
+      header->y = buffer->extent.y;
       header->width = buffer->extent.width;
       header->height = buffer->extent.height;
     }
@@ -357,7 +363,7 @@ static void
 gegl_buffer_dispose (GObject *object)
 {
   GeglBuffer  *buffer  = GEGL_BUFFER (object);
-  GeglTileHandler *handler = GEGL_HANDLER (object);
+  GeglTileHandler *handler = GEGL_TILE_HANDLER (object);
 
   gegl_buffer_sample_cleanup (buffer);
 
@@ -373,7 +379,7 @@ gegl_buffer_dispose (GObject *object)
 
   if (buffer->hot_tile)
     {
-      g_object_unref (buffer->hot_tile);
+      gegl_tile_unref (buffer->hot_tile);
       buffer->hot_tile = NULL;
     }
 
@@ -402,7 +408,7 @@ gegl_buffer_backend (GeglBuffer *buffer)
 
   do
     {
-      tmp = GEGL_HANDLER (tmp)->source;
+      tmp = GEGL_TILE_HANDLER (tmp)->source;
     } while (tmp &&
              /*GEGL_IS_TILE_TRAIT (tmp) &&*/
              !GEGL_IS_TILE_BACKEND (tmp));
@@ -458,7 +464,7 @@ gegl_buffer_constructor (GType                  type,
   object = G_OBJECT_CLASS (parent_class)->constructor (type, n_params, params);
 
   buffer    = GEGL_BUFFER (object);
-  handler   = GEGL_HANDLER (object);
+  handler   = GEGL_TILE_HANDLER (object);
   source  = handler->source;
   backend   = gegl_buffer_backend (buffer);
 
@@ -494,9 +500,7 @@ gegl_buffer_constructor (GType                  type,
           /* after construction,. x and y should be set to reflect
            * the top level behavior exhibited by this buffer object.
            */
-          g_object_set (buffer,
-                        "source", source,
-                        NULL);
+          gegl_tile_handler_set_source ((GeglTileHandler*)(buffer), source);
           g_object_unref (source);
 
           g_assert (source);
@@ -506,31 +510,19 @@ gegl_buffer_constructor (GType                  type,
       else if (buffer->path)
         {
           GeglBufferHeader *header;
-          GeglTileSource   *storage;
+          void             *storage;
 
           if (buffer->format)
-            {
-              storage = GEGL_TILE_SOURCE (g_object_new (GEGL_TYPE_TILE_STORAGE,
-                                                       "path", buffer->path,
-                                                       "format", buffer->format,
-                                                       NULL));
-            }
+            storage = gegl_tile_storage_new (-1, -1, buffer->format, buffer->path);
           else
-            {
-              storage = GEGL_TILE_SOURCE (g_object_new (GEGL_TYPE_TILE_STORAGE,
-                                                       "path", buffer->path,
-                                                       "format", babl_format ("RGBA float"),
-                                                       NULL));
-            }
+            storage = gegl_tile_storage_new (-1, -1, babl_format ("RGBA float"), buffer->path);
 
           source = g_object_new (GEGL_TYPE_BUFFER, "source", storage, NULL);
 
           /* after construction,. x and y should be set to reflect
            * the top level behavior exhibited by this buffer object.
            */
-          g_object_set (buffer,
-                        "source", source,
-                        NULL);
+          gegl_tile_handler_set_source ((GeglTileHandler*)(buffer), source);
           g_object_unref (source);
 
           g_signal_connect (storage, "changed",
@@ -558,9 +550,7 @@ gegl_buffer_constructor (GType                  type,
           /* after construction,. x and y should be set to reflect
            * the top level behavior exhibited by this buffer object.
            */
-          g_object_set (buffer,
-                        "source", source,
-                        NULL);
+          gegl_tile_handler_set_source ((GeglTileHandler*)(buffer), source);
           g_object_unref (source);
 
           g_assert (source);
@@ -576,8 +566,7 @@ gegl_buffer_constructor (GType                  type,
          * disk buffer creation, nasty but it seems to do the job.
          */
 
-        if (buffer->extent.width == 0 &&
-            buffer->extent.width == 0)
+        if (buffer->extent.width == 0)
           {
             buffer->extent.width = width;
             buffer->extent.height = height;
@@ -693,7 +682,7 @@ gegl_buffer_get_tile (GeglTileSource *source,
                       gint        y,
                       gint        z)
 {
-  GeglTileHandler *handler = GEGL_HANDLER (source);
+  GeglTileHandler *handler = GEGL_TILE_HANDLER (source);
   GeglTile    *tile   = NULL;
   source = handler->source;
 
@@ -706,6 +695,8 @@ gegl_buffer_get_tile (GeglTileSource *source,
     {
       GeglBuffer *buffer = GEGL_BUFFER (handler);
 
+      /* XXX: lock buffer? */
+
       if (x < buffer->min_x)
         buffer->min_x = x;
       if (y < buffer->min_y)
@@ -722,7 +713,12 @@ gegl_buffer_get_tile (GeglTileSource *source,
        * coordinates.
        */
       {
-        tile->tile_storage = buffer->tile_storage;
+        if (!tile->tile_storage)
+          {
+            gegl_tile_lock (tile);
+            tile->tile_storage = buffer->tile_storage;
+            gegl_tile_unlock (tile);
+          }
         tile->x = x;
         tile->y = y;
         tile->z = z;
@@ -741,13 +737,13 @@ gegl_buffer_command (GeglTileSource *source,
                      gint            z,
                      gpointer        data)
 {
-  GeglTileHandler *handler = GEGL_HANDLER (source);
+  GeglTileHandler *handler = GEGL_TILE_HANDLER (source);
   switch (command)
     {
       case GEGL_TILE_GET:
         return gegl_buffer_get_tile (source, x, y, z);
       default:
-        return gegl_tile_handler_chain_up (handler, command, x, y, z, data);
+        return gegl_tile_handler_source_command (handler, command, x, y, z, data);
     }
 }
 
@@ -755,15 +751,12 @@ static void
 gegl_buffer_class_init (GeglBufferClass *class)
 {
   GObjectClass      *gobject_class       = G_OBJECT_CLASS (class);
-  GeglTileSourceClass *tile_source_class = GEGL_TILE_SOURCE_CLASS (class);
-
   parent_class                = g_type_class_peek_parent (class);
   gobject_class->dispose      = gegl_buffer_dispose;
   gobject_class->finalize     = gegl_buffer_finalize;
   gobject_class->constructor  = gegl_buffer_constructor;
   gobject_class->set_property = gegl_buffer_set_property;
   gobject_class->get_property = gegl_buffer_get_property;
-  tile_source_class->command = gegl_buffer_command;
 
   g_object_class_install_property (gobject_class, PROP_PX_SIZE,
                                    g_param_spec_int ("px-size", "pixel-size", "size of a single pixel in bytes.",
@@ -862,12 +855,15 @@ gegl_buffer_class_init (GeglBufferClass *class)
 static gchar *
 gegl_buffer_get_alloc_stack (void)
 {
+  char  *result         = NULL;
+#ifndef HAVE_EXECINFO_H
+  result = g_strdup ("backtrace() not available for this platform\n");
+#else
   void  *functions[MAX_N_FUNCTIONS];
   int    n_functions    = 0;
   char **function_names = NULL;
   int    i              = 0;
   int    result_size    = 0;
-  char  *result         = NULL;
   
   n_functions = backtrace (functions, MAX_N_FUNCTIONS);
   function_names = backtrace_symbols (functions, n_functions);
@@ -888,11 +884,20 @@ gegl_buffer_get_alloc_stack (void)
     }
 
   free (function_names);
+#endif
 
   return result;
 }
 #endif
 
+void gegl_bt (void);
+void gegl_bt (void)
+{
+#ifdef GEGL_BUFFER_DEBUG_ALLOCATIONS
+  g_print ("%s\n", gegl_buffer_get_alloc_stack ());
+#endif
+}
+
 static void
 gegl_buffer_init (GeglBuffer *buffer)
 {
@@ -919,6 +924,7 @@ gegl_buffer_init (GeglBuffer *buffer)
   buffer->path = NULL;
   buffer->tile_width = 128;
   buffer->tile_height = 64;
+  ((GeglTileSource*)buffer)->command = gegl_buffer_command;
 
   allocated_buffers++;
 
@@ -1033,13 +1039,113 @@ gegl_buffer_destroy (GeglBuffer *buffer)
   g_object_unref (buffer);
 }
 
+typedef struct TileStorageCacheItem {
+  GeglTileStorage *storage;
+  gboolean         ram;
+  gint             tile_width;
+  gint             tile_height;
+  const void      *babl_fmt;
+} TileStorageCacheItem;
 
+static GStaticMutex storage_cache_mutex = G_STATIC_MUTEX_INIT;
+static GSList *storage_cache = NULL;
 
+/* returns TRUE if it could be done */
+gboolean gegl_tile_storage_cached_release (GeglTileStorage *storage);
+gboolean gegl_tile_storage_cached_release (GeglTileStorage *storage)
+{
+  TileStorageCacheItem *item = g_object_get_data (G_OBJECT (storage), "storage-cache-item");
 
+  if (!item)
+    return FALSE;
+  g_static_mutex_lock (&storage_cache_mutex);
+  storage_cache = g_slist_prepend (storage_cache, item);
+  g_static_mutex_unlock (&storage_cache_mutex);
+  return TRUE;
+}
 
+void gegl_tile_storage_cache_cleanup (void);
+void gegl_tile_storage_cache_cleanup (void)
+{
+  g_static_mutex_lock (&storage_cache_mutex);
+  for (;storage_cache; storage_cache = g_slist_remove (storage_cache, storage_cache->data))
+    {
+      TileStorageCacheItem *item = storage_cache->data;
+      g_object_unref (item->storage);
+    }
+  g_static_mutex_unlock (&storage_cache_mutex);
+}
+
+static GeglTileStorage *
+gegl_tile_storage_new_cached (gint tile_width, gint tile_height,
+                              const void *babl_fmt, gboolean use_ram)
+{
+  GeglTileStorage *storage = NULL;
+  GSList *iter;
+  g_static_mutex_lock (&storage_cache_mutex);
+  for (iter = storage_cache; iter; iter = iter->next)
+    {
+      TileStorageCacheItem *item = iter->data;
+      if (item->babl_fmt == babl_fmt &&
+          item->tile_width == tile_width &&
+          item->tile_height == tile_height &&
+          item->ram == item->ram)
+       {
+         storage = item->storage;
+         storage_cache = g_slist_remove (storage_cache, item);
+         break;
+       }
+    }
+
+  if (!storage)
+    {
+      TileStorageCacheItem *item = g_new0 (TileStorageCacheItem, 1);
+
+      item->tile_width = tile_width;
+      item->tile_height = tile_height;
+      item->babl_fmt = babl_fmt;
+
+      if (use_ram || 
+          !gegl_config()->swap ||
+          g_str_equal (gegl_config()->swap, "RAM") ||
+          g_str_equal (gegl_config()->swap, "ram"))
+        {
+          item->ram = TRUE;
+          storage = gegl_tile_storage_new (tile_width, tile_height, babl_fmt, NULL);
+        }
+      else
+        {
+          static gint no = 1;
+
+          gchar *filename;
+          gchar *path;
+          item->ram = FALSE;
+
+#if 0
+          filename = g_strdup_printf ("GEGL-%i-%s-%i.swap",
+                                      getpid (),
+                                      babl_name ((Babl *) babl_fmt),
+                                      no++);
+#endif
+
+          filename = g_strdup_printf ("%i-%i", getpid(), no);
+          g_atomic_int_inc (&no);
+          path = g_build_filename (gegl_config()->swap, filename, NULL);
+          g_free (filename);
+
+          storage = gegl_tile_storage_new (tile_width, tile_height, babl_fmt, path);
+          g_free (path);
+        }
+      item->storage = storage;
+      g_object_set_data_full (G_OBJECT (storage), "storage-cache-item", item, g_free);
+    }
+
+  g_static_mutex_unlock (&storage_cache_mutex);
+  return storage;
+}
 
 static GeglBuffer *
-gegl_buffer_new_from_format (const void *babl_format,
+gegl_buffer_new_from_format (const void *babl_fmt,
                              gint        x,
                              gint        y,
                              gint        width,
@@ -1051,43 +1157,7 @@ gegl_buffer_new_from_format (const void *babl_format,
   GeglTileStorage *tile_storage;
   GeglBuffer  *buffer;
 
-  if (use_ram || 
-      !gegl_config()->swap ||
-      g_str_equal (gegl_config()->swap, "RAM") ||
-      g_str_equal (gegl_config()->swap, "ram"))
-    {
-      tile_storage = g_object_new (GEGL_TYPE_TILE_STORAGE,
-                                   "format",      babl_format,
-                                   "tile-width",  tile_width,
-                                   "tile-height", tile_height,
-                                   NULL);
-    }
-  else
-    {
-      static gint no = 1;
-
-      gchar *filename;
-      gchar *path;
-
-#if 0
-      filename = g_strdup_printf ("GEGL-%i-%s-%i.swap",
-                                  getpid (),
-                                  babl_name ((Babl *) babl_format),
-                                  no++);
-#endif
-
-      filename = g_strdup_printf ("%i-%i", getpid(), no++);
-      path = g_build_filename (gegl_config()->swap, filename, NULL);
-      g_free (filename);
-
-      tile_storage = g_object_new (GEGL_TYPE_TILE_STORAGE,
-                                   "format",      babl_format,
-                                   "tile-width",  tile_width,
-                                   "tile-height", tile_height,
-                                   "path",        path,
-                                   NULL);
-      g_free (path);
-    }
+  tile_storage = gegl_tile_storage_new_cached (tile_width, tile_height, babl_fmt, use_ram);
 
   buffer = g_object_new (GEGL_TYPE_BUFFER,
                          "source",      tile_storage,
@@ -1122,6 +1192,7 @@ gegl_buffer_void (GeglBuffer *buffer)
   gint tile_height = buffer->tile_storage->tile_height;
   gint bufy        = 0;
 
+  g_mutex_lock (buffer->tile_storage->mutex);
   {
     gint z;
     gint factor = 1;
@@ -1165,6 +1236,7 @@ done_with_row:
         factor *= 2;
       }
   }
+  g_mutex_unlock (buffer->tile_storage->mutex);
 }
 
 
@@ -1181,12 +1253,13 @@ gboolean gegl_buffer_is_shared (GeglBuffer *buffer)
 
 gboolean gegl_buffer_try_lock (GeglBuffer *buffer)
 {
-  GeglTileBackend *backend = gegl_buffer_backend (buffer);
   gboolean ret;
-
+  GeglTileBackend *backend = gegl_buffer_backend (buffer);
+  g_mutex_lock (buffer->tile_storage->mutex);
   if (buffer->lock_count>0)
     {
       buffer->lock_count++;
+      g_mutex_unlock (buffer->tile_storage->mutex);
       return TRUE;
     }
   if (gegl_buffer_is_shared(buffer))
@@ -1195,15 +1268,16 @@ gboolean gegl_buffer_try_lock (GeglBuffer *buffer)
     ret = TRUE;
   if (ret)
     buffer->lock_count++;
+  g_mutex_unlock (buffer->tile_storage->mutex);
   return TRUE;
 }
 
-#if 0
+/* this locking is for synchronising shared buffers */
 gboolean gegl_buffer_lock (GeglBuffer *buffer)
 {
   while (gegl_buffer_try_lock (buffer)==FALSE)
     {
-      g_print ("failed to aquire lock blocking ..");
+      g_print ("waiting to aquire buffer..");
         g_usleep (100000);
     }
   return TRUE;
@@ -1211,12 +1285,16 @@ gboolean gegl_buffer_lock (GeglBuffer *buffer)
 
 gboolean gegl_buffer_unlock (GeglBuffer *buffer)
 {
+  gboolean ret = TRUE;
   GeglTileBackend *backend = gegl_buffer_backend (buffer);
-  g_assert (buffer->lock_count>=0);
+  g_mutex_lock (buffer->tile_storage->mutex);
+  g_assert (buffer->lock_count >=0);
   buffer->lock_count--;
-  g_assert (buffer->lock_count>=0);
-  if (buffer->lock_count==0 && gegl_buffer_is_shared (buffer))
-    return gegl_tile_backend_file_unlock (GEGL_TILE_BACKEND_FILE (backend));
-  return TRUE;
+  g_assert (buffer->lock_count >=0);
+  if (buffer->lock_count == 0 && gegl_buffer_is_shared (buffer))
+    {
+      ret = gegl_tile_backend_file_unlock (GEGL_TILE_BACKEND_FILE (backend));
+    }
+  g_mutex_unlock (buffer->tile_storage->mutex);
+  return ret;
 }
-#endif
