@@ -15,11 +15,16 @@
  *
  */
 
+#include "config.h"
+#include <string.h>
+
 #include <glib-object.h>
-#include "gegl-types.h"
+
+#include "gegl.h"
+#include "gegl-types-internal.h"
 #include "gegl-buffer-private.h"
 #include "gegl-sampler-nearest.h"
-#include <string.h>
+
 
 enum
 {
@@ -58,7 +63,10 @@ gegl_sampler_nearest_class_init (GeglSamplerNearestClass *klass)
 static void
 gegl_sampler_nearest_init (GeglSamplerNearest *self)
 {
-   GEGL_SAMPLER (self)->context_rect = (GeglRectangle){0,0,1,1};
+   GEGL_SAMPLER (self)->context_rect.x = 0;
+   GEGL_SAMPLER (self)->context_rect.y = 0;
+   GEGL_SAMPLER (self)->context_rect.width = 1;
+   GEGL_SAMPLER (self)->context_rect.height = 1;
    GEGL_SAMPLER (self)->interpolate_format = babl_format ("RGBA float");
 }
 
