@@ -45,7 +45,7 @@ struct _RamEntry
   guchar *offset;
 };
 
-static void inline
+static inline void
 ram_entry_read (GeglTileBackendRam *ram,
                 RamEntry           *entry,
                 guchar             *dest)
@@ -55,7 +55,7 @@ ram_entry_read (GeglTileBackendRam *ram,
   memcpy (dest, entry->offset, tile_size);
 }
 
-static void inline
+static inline void
 ram_entry_write (GeglTileBackendRam *ram,
                  RamEntry           *entry,
                  guchar             *source)
@@ -155,11 +155,9 @@ get_tile (GeglTileSource *tile_store,
     if (!entry)
       return NULL;
 
-    tile             = gegl_tile_new (backend->tile_size);
-    tile->stored_rev = 1;
-    tile->rev        = 1;
+    tile = gegl_tile_new (backend->tile_size);
 
-    ram_entry_read (tile_backend_ram, entry, tile->data);
+    ram_entry_read (tile_backend_ram, entry, gegl_tile_get_data (tile));
   }
   return tile;
 }
@@ -184,8 +182,8 @@ gboolean set_tile (GeglTileSource *store,
       entry->z = z;
       g_hash_table_insert (tile_backend_ram->entries, entry, entry);
     }
-  ram_entry_write (tile_backend_ram, entry, tile->data);
-  tile->stored_rev = tile->rev;
+  ram_entry_write (tile_backend_ram, entry, gegl_tile_get_data (tile));
+  gegl_tile_mark_as_stored (tile);
   return TRUE;
 }
 
@@ -253,7 +251,7 @@ gegl_tile_backend_ram_command (GeglTileSource     *tile_store,
         return NULL;
 
       case GEGL_TILE_EXIST:
-        return (gpointer)exist_tile (tile_store, data, x, y, z);
+        return GINT_TO_POINTER(exist_tile (tile_store, data, x, y, z));
 
       default:
         g_assert (command < GEGL_TILE_LAST_COMMAND &&
@@ -358,7 +356,6 @@ static void
 gegl_tile_backend_ram_class_init (GeglTileBackendRamClass *klass)
 {
   GObjectClass    *gobject_class     = G_OBJECT_CLASS (klass);
-  GeglTileSourceClass *gegl_tile_source_class = GEGL_TILE_SOURCE_CLASS (klass);
 
   parent_class = g_type_class_peek_parent (klass);
 
@@ -366,12 +363,11 @@ gegl_tile_backend_ram_class_init (GeglTileBackendRamClass *klass)
   gobject_class->set_property = set_property;
   gobject_class->constructor  = gegl_tile_backend_ram_constructor;
   gobject_class->finalize     = finalize;
-
-  gegl_tile_source_class->command  = gegl_tile_backend_ram_command;
 }
 
 static void
 gegl_tile_backend_ram_init (GeglTileBackendRam *self)
 {
+  ((GeglTileSource*)self)->command = gegl_tile_backend_ram_command;
   self->entries = NULL;
 }
