@@ -26,8 +26,6 @@
 
 G_DEFINE_TYPE (GeglTileHandlerChain, gegl_tile_handler_chain, GEGL_TYPE_TILE_HANDLER)
 
-static void   gegl_tile_handler_chain_rebind (GeglTileHandlerChain *tile_handler_chain);
-
 static void
 gegl_tile_handler_chain_nuke_cache (GeglTileHandlerChain *tile_handler_chain)
 {
@@ -42,7 +40,6 @@ gegl_tile_handler_chain_nuke_cache (GeglTileHandlerChain *tile_handler_chain)
             {
               g_object_unref (iter->data);
               tile_handler_chain->chain = g_slist_remove (tile_handler_chain->chain, iter->data);
-              gegl_tile_handler_chain_rebind (tile_handler_chain);
               break;
             }
           iter = iter->next;
@@ -51,10 +48,11 @@ gegl_tile_handler_chain_nuke_cache (GeglTileHandlerChain *tile_handler_chain)
 }
 
 static void
-dispose (GObject *object)
+gegl_tile_handler_chain_dispose (GObject *object)
 {
   GeglTileHandlerChain *tile_handler_chain = GEGL_TILE_HANDLER_CHAIN (object);
   GSList       *iter;
+  GSList       *chain;
 
   /* Get rid of the cache before any further parts of the deconstruction of the
    * TileStore chain, unwritten tiles need a living TileStore for their
@@ -62,24 +60,36 @@ dispose (GObject *object)
    */
   gegl_tile_handler_chain_nuke_cache (tile_handler_chain);
 
-  iter = tile_handler_chain->chain;
+  /* While we destroy tile_handler_chain->chain, don't have
+   * tile_handler_chain->chain set, because it might otherwise be used
+   * by other objects in the dispose phase
+   */
+  chain = tile_handler_chain->chain;
+  tile_handler_chain->chain = NULL;
+  
+  iter = chain;
   while (iter)
     {
       if (iter->data)
-        g_object_unref (iter->data);
+        {
+          if (G_IS_OBJECT (iter->data))
+            g_object_unref (iter->data);
+          else
+            g_free (iter->data);
+        }
       iter = iter->next;
     }
 
-  if (tile_handler_chain->chain)
-    g_slist_free (tile_handler_chain->chain);
-  tile_handler_chain->chain = NULL;
+  if (chain)
+    g_slist_free (chain);
+  chain = NULL;
 
   G_OBJECT_CLASS (gegl_tile_handler_chain_parent_class)->dispose (object);
 }
 
 
 static void
-finalize (GObject *object)
+gegl_tile_handler_chain_finalize (GObject *object)
 {
   G_OBJECT_CLASS (gegl_tile_handler_chain_parent_class)->finalize (object);
 }
@@ -110,30 +120,24 @@ static void
 gegl_tile_handler_chain_class_init (GeglTileHandlerChainClass *class)
 {
   GObjectClass      *gobject_class;
-  GeglTileSourceClass *tile_store_class;
-
   gobject_class    = (GObjectClass *) class;
-  tile_store_class = (GeglTileSourceClass *) class;
 
-  tile_store_class->command = gegl_tile_handler_chain_command;
-
-  gobject_class->finalize = finalize;
-  gobject_class->dispose  = dispose;
+  gobject_class->finalize = gegl_tile_handler_chain_finalize;
+  gobject_class->dispose  = gegl_tile_handler_chain_dispose;
 }
 
 static void
 gegl_tile_handler_chain_init (GeglTileHandlerChain *self)
 {
+  ((GeglTileSource*)self)->command = gegl_tile_handler_chain_command;
   self->chain = NULL;
 }
 
-
-static void
-gegl_tile_handler_chain_rebind (GeglTileHandlerChain *tile_handler_chain)
+void
+gegl_tile_handler_chain_bind (GeglTileHandlerChain *tile_handler_chain)
 {
   GSList *iter;
 
-
   iter = tile_handler_chain->chain;
   while (iter)
     {
@@ -159,7 +163,7 @@ gegl_tile_handler_chain_add (GeglTileHandlerChain *tile_handler_chain,
                              GeglTileHandler      *handler)
 {
   tile_handler_chain->chain = g_slist_prepend (tile_handler_chain->chain, handler);
-  gegl_tile_handler_chain_rebind (tile_handler_chain);
+  /*gegl_tile_handler_chain_bind (tile_handler_chain);*/
   return handler;
 }
 
