@@ -26,7 +26,7 @@
 G_DEFINE_TYPE (GeglTileHandlerLog, gegl_tile_handler_log, GEGL_TYPE_TILE_HANDLER)
 
 
-static char *commands[] =
+static const char *commands[] =
 {
   "idle",
   "set",
@@ -49,10 +49,10 @@ gegl_tile_handler_log_command (GeglTileSource  *gegl_tile_source,
                                gint             z,
                                gpointer         data)
 {
-  GeglTileHandler *handler = GEGL_HANDLER (gegl_tile_source);
+  GeglTileHandler *handler = GEGL_TILE_HANDLER (gegl_tile_source);
   gpointer         result = NULL;
 
-  result = gegl_tile_handler_chain_up (handler, command, x, y, z, data);
+  result = gegl_tile_handler_source_command (handler, command, x, y, z, data);
 
   switch (command)
     {
@@ -62,11 +62,11 @@ gegl_tile_handler_log_command (GeglTileSource  *gegl_tile_source,
       default:
         if (result)
         g_print ("(%s %p %p %i·%i·%i ⇒%p)", 
-          commands[command], (void *) ((gint)gegl_tile_source&0xffff), (void*)((gint)data&0xffff), x, y, z,
+          commands[command], GINT_TO_POINTER(GPOINTER_TO_INT(gegl_tile_source)&0xffff), GINT_TO_POINTER(GPOINTER_TO_INT(data)&0xffff), x, y, z,
           result);
         else
         g_print ("(%s %p %p %i·%i·%i ☹)", 
-          commands[command], (void *) ((gint)gegl_tile_source&0xffff), data, x, y, z);
+          commands[command], GINT_TO_POINTER(GPOINTER_TO_INT(gegl_tile_source)&0xffff), data, x, y, z);
     }
   return result;
 }
@@ -74,12 +74,10 @@ gegl_tile_handler_log_command (GeglTileSource  *gegl_tile_source,
 static void
 gegl_tile_handler_log_class_init (GeglTileHandlerLogClass *klass)
 {
-  GeglTileSourceClass *source_class = GEGL_TILE_SOURCE_CLASS (klass);
-
-  source_class->command = gegl_tile_handler_log_command;
 }
 
 static void
 gegl_tile_handler_log_init (GeglTileHandlerLog *self)
 {
+  ((GeglTileSource*)self)->command = gegl_tile_handler_log_command;
 }
