@@ -27,13 +27,6 @@
 
 G_DEFINE_TYPE (GeglTileHandlerZoom, gegl_tile_handler_zoom, GEGL_TYPE_TILE_HANDLER)
 
-enum
-{
-  PROP_0,
-  PROP_TILE_STORAGE,
-  PROP_BACKEND
-};
-
 #include <babl/babl.h>
 #include "gegl-tile-backend.h"
 #include "gegl-tile-storage.h"
@@ -168,7 +161,7 @@ downscale_u8 (gint    components,
     }
 }
 
-static void inline set_half (GeglTile * dst_tile,
+static inline void set_half (GeglTile * dst_tile,
                              GeglTile * src_tile,
                              gint       width,
                              gint       height,
@@ -204,8 +197,8 @@ get_tile (GeglTileSource *gegl_tile_source,
           gint            y,
           gint            z)
 {
-  GeglTileSource      *source = GEGL_HANDLER (gegl_tile_source)->source;
-  GeglTileHandlerZoom *zoom   = GEGL_TILE_HANDLER_ZOOM (gegl_tile_source);
+  GeglTileSource      *source = ((GeglTileHandler*)(gegl_tile_source))->source;
+  GeglTileHandlerZoom *zoom   = (GeglTileHandlerZoom*)(gegl_tile_source);
   GeglTile            *tile   = NULL;
   Babl                *format = (Babl *) (zoom->backend->format);
   gint                 tile_width;
@@ -237,7 +230,6 @@ get_tile (GeglTileSource *gegl_tile_source,
 
   {
     gint      i, j;
-    guchar   *data;
     GeglTile *source_tile[2][2] = { { NULL, NULL }, { NULL, NULL } };
 
     for (i = 0; i < 2; i++)
@@ -267,20 +259,11 @@ get_tile (GeglTileSource *gegl_tile_source,
         tile->y          = y;
         tile->z          = z;
         tile->tile_storage = zoom->tile_storage;
-        tile->stored_rev = 1;
-        tile->rev        = 1;
 
-        {
-          GeglTileHandlerCache *cache;
-          cache = g_object_get_data (G_OBJECT (gegl_tile_source), "cache");
-          if (cache)
-            {
-              gegl_tile_handler_cache_insert (cache, tile, x, y, z);
-            }
-        }
+        if (zoom->cache)
+          gegl_tile_handler_cache_insert (zoom->cache, tile, x, y, z);
       }
     gegl_tile_lock (tile);
-    data = gegl_tile_get_data (tile);
 
     for (i = 0; i < 2; i++)
       for (j = 0; j < 2; j++)
@@ -288,7 +271,7 @@ get_tile (GeglTileSource *gegl_tile_source,
           if (source_tile[i][j])
             {
               set_half (tile, source_tile[i][j], tile_width, tile_height, format, i, j);
-              g_object_unref (source_tile[i][j]);
+              gegl_tile_unref (source_tile[i][j]);
             }
           else 
             {
@@ -308,118 +291,37 @@ gegl_tile_handler_zoom_command (GeglTileSource  *tile_store,
                                 gint             y,
                                 gint             z,
                                 gpointer         data)
-  {
-  GeglTileHandler *handler  = GEGL_HANDLER (tile_store);
-  /*GeglTileSource *source = handler->source;*/
-
-  switch (command)
-    {
-      case GEGL_TILE_GET:
-        return get_tile (tile_store, x, y, z);
-      default:
-        /* pass the command on */
-        return gegl_tile_handler_chain_up (handler, command, x, y, z, data);
-    }
-}
-
-
-static void
-get_property (GObject    *gobject,
-              guint       property_id,
-              GValue     *value,
-              GParamSpec *pspec)
 {
-  GeglTileHandlerZoom *zoom = GEGL_TILE_HANDLER_ZOOM (gobject);
+  GeglTileHandler *handler  = (void*)tile_store;
 
-  switch (property_id)
-    {
-      case PROP_TILE_STORAGE:
-        g_value_set_object (value, zoom->tile_storage);
-        break;
-
-      case PROP_BACKEND:
-        g_value_set_object (value, zoom->backend);
-        break;
-
-      default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
-        break;
-    }
+  if (command == GEGL_TILE_GET)
+    return get_tile (tile_store, x, y, z);
+  else
+    return gegl_tile_handler_source_command (handler, command, x, y, z, data);
 }
 
-static void
-set_property (GObject      *gobject,
-              guint         property_id,
-              const GValue *value,
-              GParamSpec   *pspec)
-{
-  GeglTileHandlerZoom *zoom = GEGL_TILE_HANDLER_ZOOM (gobject);
-
-  switch (property_id)
-    {
-      case PROP_TILE_STORAGE:
-        zoom->tile_storage = g_value_get_object (value);
-        break;
-
-      case PROP_BACKEND:
-        zoom->backend = g_value_get_object (value);
-        break;
-
-      default:
-        G_OBJECT_WARN_INVALID_PROPERTY_ID (gobject, property_id, pspec);
-        break;
-    }
-}
-
-static GObject *
-constructor (GType                  type,
-             guint                  n_params,
-             GObjectConstructParam *params)
-{
-  GObject      *object;
-  GeglTileHandlerZoom *zoom;
-
-  object = G_OBJECT_CLASS (gegl_tile_handler_zoom_parent_class)->constructor (type, n_params, params);
-
-  zoom   = GEGL_TILE_HANDLER_ZOOM (object);
-
-  return object;
-}
-
-
 static void
 gegl_tile_handler_zoom_class_init (GeglTileHandlerZoomClass *klass)
 {
-  GObjectClass      *gobject_class  = G_OBJECT_CLASS (klass);
-  GeglTileSourceClass *source_class = GEGL_TILE_SOURCE_CLASS (klass);
-
-  gobject_class->constructor  = constructor;
-  gobject_class->set_property = set_property;
-  gobject_class->get_property = get_property;
-
-  source_class->command = gegl_tile_handler_zoom_command;
-
-  g_object_class_install_property (gobject_class, PROP_TILE_STORAGE,
-                                   g_param_spec_object ("tile_storage",
-                                                        "tile_storage",
-                                                        "tile_storage for this tilestore (needed for tile size data)",
-                                                        G_TYPE_OBJECT,
-                                                        G_PARAM_WRITABLE |
-                                                        G_PARAM_CONSTRUCT_ONLY));
-
-
-  g_object_class_install_property (gobject_class, PROP_BACKEND,
-                                   g_param_spec_object ("backend",
-                                                        "backend",
-                                                        "backend for this tilestore (needed for tile size data)",
-                                                        G_TYPE_OBJECT,
-                                                        G_PARAM_WRITABLE |
-                                                        G_PARAM_CONSTRUCT_ONLY));
 }
 
 static void
 gegl_tile_handler_zoom_init (GeglTileHandlerZoom *self)
 {
+  ((GeglTileSource*)self)->command = gegl_tile_handler_zoom_command;
   self->backend = NULL;
   self->tile_storage = NULL;
 }
+
+GeglTileHandler *
+gegl_tile_handler_zoom_new (GeglTileBackend      *backend,
+                            GeglTileStorage      *tile_storage,
+                            GeglTileHandlerCache *cache)
+{
+  GeglTileHandlerZoom *ret = g_object_new (GEGL_TYPE_TILE_HANDLER_ZOOM, NULL);
+  ((GeglTileSource*)ret)->command = gegl_tile_handler_zoom_command;
+  ret->backend = backend;
+  ret->tile_storage = tile_storage;
+  ret->cache = cache;
+  return (void*)ret;
+}
