@@ -49,9 +49,7 @@ struct _GeglTile
   gchar            lock;        /* number of times the tile is write locked
                                  * should in theory just have the values 0/1
                                  */
-#if ENABLE_MT
   GMutex          *mutex;
-#endif
 
   /* the shared list is a doubly linked circular list */
   GeglTile        *next_shared;
@@ -62,32 +60,40 @@ struct _GeglTile
   gpointer         destroy_notify_data;
 };
 
-GeglTile   * gegl_tile_new        (gint     size);
-GeglTile   * gegl_tile_new_bare   (void); /* special hack for linear bufs */
-GeglTile   * gegl_tile_ref        (GeglTile *tile);
-void         gegl_tile_unref      (GeglTile *tile);
-void       * gegl_tile_get_format (GeglTile *tile);
+GeglTile   * gegl_tile_new            (gint     size);
+GeglTile   * gegl_tile_new_bare       (void); /* special hack for linear bufs */
+GeglTile   * gegl_tile_ref            (GeglTile *tile);
+void         gegl_tile_unref          (GeglTile *tile);
+void       * gegl_tile_get_format     (GeglTile *tile);
 
 
 /* lock a tile for writing, this would allow writing to buffers
  * later gotten with get_data()
  */
-void         gegl_tile_lock       (GeglTile *tile);
-/* get a pointer to the linear buffer of the tile.
- */
-#define gegl_tile_get_data(tile)  ((guchar*)((tile)->data))
+void         gegl_tile_lock           (GeglTile *tile);
 
 /* unlock the tile notifying the tile that we're done manipulating
  * the data.
  */
-void         gegl_tile_unlock     (GeglTile *tile);
+void         gegl_tile_unlock         (GeglTile *tile);
 
 
 
-gboolean     gegl_tile_is_stored  (GeglTile *tile);
-gboolean     gegl_tile_store      (GeglTile *tile);
-void         gegl_tile_void       (GeglTile *tile);
-GeglTile    *gegl_tile_dup        (GeglTile *tile);
+void         gegl_tile_mark_as_stored (GeglTile *tile);
+gboolean     gegl_tile_is_stored      (GeglTile *tile);
+gboolean     gegl_tile_store          (GeglTile *tile);
+void         gegl_tile_void           (GeglTile *tile);
+GeglTile    *gegl_tile_dup            (GeglTile *tile);
+
+/* utility low-level functions used by an undo system in horizon
+ * where the geglbufer originated, kept around in case they
+ * become useful again
+ */
+void         gegl_tile_swp            (GeglTile *a,
+                                       GeglTile *b);
+void         gegl_tile_cpy            (GeglTile *src,
+                                       GeglTile *dst);
+
 
 /* computes the positive integer remainder (also for negative dividends)
  */
@@ -96,6 +102,10 @@ GeglTile    *gegl_tile_dup        (GeglTile *tile);
                     (divisor) - 1 - ((-((dividend) + 1)) % (divisor)) : \
                     (dividend) % (divisor))
 
+/* get a pointer to the linear buffer of the tile.
+ */
+#define gegl_tile_get_data(tile)  ((guchar*)((tile)->data))
+
 #define gegl_tile_offset(coordinate, stride) GEGL_REMAINDER((coordinate), (stride))
 
 /* helper function to compute tile indices and offsets for coordinates
@@ -106,13 +116,5 @@ GeglTile    *gegl_tile_dup        (GeglTile *tile);
       (coordinate) / (stride):\
       ((((coordinate) + 1) /(stride)) - 1))
 
-/* utility low-level functions used by an undo system in horizon
- * where the geglbufer originated, kept around in case they
- * become useful again
- */
-void         gegl_tile_swp        (GeglTile *a,
-                                   GeglTile *b);
-void         gegl_tile_cpy        (GeglTile *src,
-                                   GeglTile *dst);
 
 #endif
