@@ -39,6 +39,22 @@
 
 #include <windows.h>
 
+static HMODULE hLibGeglModule = NULL;
+
+/* DllMain prototype */
+BOOL WINAPI DllMain (HINSTANCE hinstDLL, 
+                     DWORD     fdwReason, 
+                     LPVOID    lpvReserved);
+
+BOOL WINAPI
+DllMain (HINSTANCE hinstDLL, 
+         DWORD     fdwReason, 
+         LPVOID    lpvReserved)
+{
+  hLibGeglModule = hinstDLL;
+  return TRUE;
+}
+
 static inline gboolean
 pid_is_running (gint pid)
 {
@@ -77,6 +93,7 @@ guint gegl_debug_flags = 0;
 #include "operation/gegl-extension-handler.h"
 #include "buffer/gegl-buffer-private.h"
 #include "gegl-config.h"
+#include "graph/gegl-node.h"
 
 
 /* if this function is made to return NULL swapping is disabled */
@@ -189,6 +206,7 @@ static gchar   *cmd_gegl_chunk_size=NULL;
 static gchar   *cmd_gegl_quality=NULL;
 static gchar   *cmd_gegl_tile_size=NULL;
 static gchar   *cmd_babl_tolerance =NULL;
+static gchar   *cmd_gegl_threads=NULL;
 
 static const GOptionEntry cmd_entries[]=
 {
@@ -222,6 +240,11 @@ static const GOptionEntry cmd_entries[]=
      G_OPTION_ARG_STRING, &cmd_gegl_quality,
      N_("The quality of rendering a value between 0.0(fast) and 1.0(reference)"), "<quality>"
     },
+    {
+     "gegl-threads", 0, 0,
+     G_OPTION_ARG_STRING, &cmd_gegl_threads,
+     N_("The number of concurrent processing threads to use."), "<threads>"
+    },
     { NULL }
 };
 
@@ -268,6 +291,16 @@ GeglConfig *gegl_config (void)
           if (str)
             config->tile_height = atoi(str+1);
         }
+      if (g_getenv ("GEGL_THREADS"))
+        {
+          config->threads = atoi(g_getenv("GEGL_THREADS"));
+          if (config->threads > GEGL_MAX_THREADS)
+            {
+              g_warning ("Tried to use %i threads max is %i",
+                         config->threads, GEGL_MAX_THREADS);
+              config->threads = GEGL_MAX_THREADS;
+            }
+        }
       if (gegl_swap_dir())
         config->swap = g_strdup(gegl_swap_dir ());
     }
@@ -318,12 +351,14 @@ static void swap_clean (void)
     }
 }
 
+void gegl_tile_storage_cache_cleanup (void);
 
 void
 gegl_exit (void)
 {
   glong timing = gegl_ticks ();
 
+  gegl_tile_storage_cache_cleanup ();
   gegl_tile_cache_destroy ();
   gegl_operation_gtype_cleanup ();
   gegl_extension_handler_cleanup ();
@@ -358,7 +393,7 @@ gegl_exit (void)
     }
 
   if (gegl_buffer_leaks ())
-    g_printf ("  buffer-leaks: %i", gegl_buffer_leaks ());
+    g_printf ("  buffer-leaks: %i\n", gegl_buffer_leaks ());
   gegl_tile_cache_destroy ();
 
   if (gegl_swap_dir ())
@@ -395,8 +430,6 @@ gegl_exit (void)
     }
   g_object_unref (config);
   config = NULL;
-
-  g_printf ("\n");
 }
 
 static void swap_clean (void);
@@ -460,7 +493,8 @@ gegl_post_parse_hook (GOptionContext *context,
       if (str)
         config->tile_height = atoi(str+1);
     }
-
+  if (cmd_gegl_threads)
+    config->threads = atoi (cmd_gegl_threads);
   if (cmd_babl_tolerance)
     g_object_set (config, "babl-tolerance", atof(cmd_babl_tolerance), NULL);
 
@@ -502,10 +536,12 @@ gegl_post_parse_hook (GOptionContext *context,
           gchar *module_path;
 
 #ifdef G_OS_WIN32
-          module_path =
-            g_win32_get_package_installation_subdirectory (NULL,
-                                                           "lib" GEGL_LIBRARY "-0.dll",
-                                                           "lib/" GEGL_LIBRARY);
+          {
+            gchar *prefix;
+            prefix = g_win32_get_package_installation_directory_of_module ( hLibGeglModule );
+            module_path = g_build_filename (prefix, "lib", GEGL_LIBRARY, NULL);
+            g_free(prefix);
+          }
 #else
           module_path = g_build_filename (LIBDIR, GEGL_LIBRARY, NULL);
 #endif
@@ -616,7 +652,7 @@ static const gchar *makefile (void)
     "# add a dedicated target with the extra bits linked in.\n"
     "\n\n"
     "CFLAGS  += `pkg-config gegl --cflags`  -I. -fPIC\n"
-    /*"LDFLAGS += `pkg-config --libs` -shared\n"*/
+    "LDFLAGS += `pkg-config gegl --libs` -shared\n"
     "SHREXT=.so\n"
     "CFILES = $(wildcard ./*.c)\n"
     "SOBJS  = $(subst ./,,$(CFILES:.c=$(SHREXT)))\n"
