@@ -87,6 +87,16 @@ gboolean    gegl_rectangle_equal_coords  (const GeglRectangle *rectangle,
                                           gint                 width,
                                           gint                 height);
 
+/**
+ * gegl_rectangle_is_empty:
+ * @rectangle: a #GeglRectangle
+ *
+ * Check if a rectangle has zero area.
+ *
+ * Returns TRUE if @rectangle height and width are both zero.
+ */
+gboolean    gegl_rectangle_is_empty     (const GeglRectangle *rectangle);
+
 /**
  * gegl_rectangle_copy:
  * @destination: a #GeglRectangle
@@ -196,14 +206,12 @@ void     gegl_free                    (gpointer mem);
 
 #define INT_MULT(a,b,t)  ((t) = (a) * (b) + 0x80, ((((t) >> 8) + (t)) >> 8))
 
+/***
+ */
 inline gint _gegl_float_epsilon_zero  (float     value);
 gint        _gegl_float_epsilon_equal (float     v1,
                                        float     v2);
 
-
-/***
- */
-
 G_END_DECLS
 
 #endif /* __GEGL_UTILS_H__ */
