@@ -61,7 +61,6 @@ enum
   LAST_SIGNAL
 };
 
-#define MAX_THREADS 8
 
 struct _GeglNodePrivate
 {
@@ -72,7 +71,7 @@ struct _GeglNodePrivate
   gchar          *name;
   GeglProcessor  *processor;
   GHashTable     *contexts;
-  GeglEvalMgr    *eval_mgr[MAX_THREADS];
+  GeglEvalMgr    *eval_mgr[GEGL_MAX_THREADS];
 };
 
 
@@ -200,9 +199,7 @@ gegl_node_init (GeglNode *self)
   self->operation      = NULL;
   self->is_graph       = FALSE;
   self->cache          = NULL;
-#if ENABLE_MT
   self->mutex          = g_mutex_new ();
-#endif
 
 }
 
@@ -282,9 +279,7 @@ gegl_node_finalize (GObject *gobject)
       g_free (self->priv->name);
     }
   g_hash_table_destroy (self->priv->contexts);
-#if ENABLE_MT
   g_mutex_free (self->mutex);
-#endif
 
   G_OBJECT_CLASS (gegl_node_parent_class)->finalize (gobject);
 }
@@ -394,6 +389,13 @@ gegl_node_get_pad (GeglNode    *self,
   return NULL;
 }
 
+gboolean
+gegl_node_has_pad (GeglNode      *self,
+                   const gchar   *name)
+{
+  return gegl_node_get_pad (self, name) != NULL;
+}
+
 /**
  * gegl_node_get_pads:
  * @self: a #GeglNode.
@@ -849,7 +851,7 @@ gegl_node_apply_roi (GeglNode            *self,
    */
   GeglBuffer *buffer;
 
-  //g_print ("%i %i %i %i %i\n", tid, roi->x, roi->y, roi->width, roi->height);
+  /*g_print ("%i %i %i %i %i\n", tid, roi->x, roi->y, roi->width, roi->height);*/
 
   if (roi)
     {
@@ -864,7 +866,6 @@ gegl_node_apply_roi (GeglNode            *self,
 }
 
 
-#if ENABLE_MT
 typedef struct ThreadData
 {
   GeglNode      *node;
@@ -908,7 +909,6 @@ static void spawnrender (gpointer data,
     }
   g_mutex_unlock (mutex);
 }
-#endif
 
 
 void
@@ -920,15 +920,13 @@ gegl_node_blit (GeglNode            *self,
                 gint                 rowstride,
                 GeglBlitFlags        flags)
 {
-#if ENABLE_MT
   gint threads;
-#endif
   g_return_if_fail (GEGL_IS_NODE (self));
   g_return_if_fail (roi != NULL);
 
-#if ENABLE_MT
+#if 1
   threads = gegl_config ()->threads;
-  if (threads > MAX_THREADS)
+  if (threads > GEGL_MAX_THREADS)
     threads = 0;
   
   if (pool == NULL)
@@ -949,7 +947,7 @@ gegl_node_blit (GeglNode            *self,
                               */
   if (flags == GEGL_BLIT_DEFAULT)
     {
-      ThreadData data[MAX_THREADS];
+      ThreadData data[GEGL_MAX_THREADS];
       gint i;
 
       /* Subdivide along the largest of width/height, this should be further
@@ -1024,7 +1022,7 @@ gegl_node_blit (GeglNode            *self,
           g_mutex_unlock (mutex);
         }
     }
-#else
+#else  /* thread free version, can be removed */
     if (flags == GEGL_BLIT_DEFAULT)
     {
       GeglBuffer *buffer;
@@ -1250,6 +1248,7 @@ gegl_node_set_operation_object (GeglNode      *self,
     GSList   *output_c        = NULL;
     GeglNode *output          = NULL;
     gchar    *output_dest_pad = NULL;
+    GSList   *old_pads        = NULL;
     GeglNode *input           = NULL;
     GeglNode *aux             = NULL;
 
@@ -1278,19 +1277,17 @@ gegl_node_set_operation_object (GeglNode      *self,
     gegl_node_disconnect_sources (self);
     gegl_node_disconnect_sinks (self);
 
-    /* FIXME: handle this in a more generic way, but it is needed to allow
-     * the attach to work properly.
-     */
-
-    if (gegl_node_get_pad (self, "output"))
-      gegl_node_remove_pad (self, gegl_node_get_pad (self, "output"));
-    if (gegl_node_get_pad (self, "input"))
-      gegl_node_remove_pad (self, gegl_node_get_pad (self, "input"));
-    if (gegl_node_get_pad (self, "aux"))
-      gegl_node_remove_pad (self, gegl_node_get_pad (self, "aux"));
+    /* Delete all the pads from the previous operation */
+    while ((old_pads = gegl_node_get_pads (self)) != NULL) 
+      {
+        gegl_node_remove_pad (self, old_pads->data);
+      }
 
     gegl_operation_attach (operation, self);
 
+    /* FIXME: handle this in a more generic way, but it is needed to allow
+     * the attach to work properly.
+     */
     if (input)
       gegl_node_connect_from (self, "input", input, "output");
     if (aux)
@@ -1802,14 +1799,10 @@ gegl_node_get_context (GeglNode *self,
                        gpointer  context_id)
 {
   GeglOperationContext *context = NULL;
-#if ENABLE_MT
   g_mutex_lock (self->mutex);
-#endif
 
   context = g_hash_table_lookup (self->priv->contexts, context_id);
-#if ENABLE_MT
   g_mutex_unlock (self->mutex);
-#endif
   return context;
 }
 
@@ -1823,23 +1816,17 @@ gegl_node_remove_context (GeglNode *self,
   g_return_if_fail (context_id != NULL);
 
   context = gegl_node_get_context (self, context_id);
-#if ENABLE_MT
   g_mutex_lock (self->mutex);
-#endif
   if (!context)
     {
       g_warning ("didn't find context %p for %s",
                  context_id, gegl_node_get_debug_name (self));
-#if ENABLE_MT
       g_mutex_unlock (self->mutex);
-#endif
       return;
     }
   g_hash_table_remove (self->priv->contexts, context_id);
   gegl_operation_context_destroy (context);
-#if ENABLE_MT
   g_mutex_unlock (self->mutex);
-#endif
 }
 
 /* Creates, sets up and returns a new context for the node, or just returns it
@@ -1854,27 +1841,21 @@ gegl_node_add_context (GeglNode *self,
   g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
   g_return_val_if_fail (context_id != NULL, NULL);
 
-#if ENABLE_MT
   g_mutex_lock (self->mutex);
-#endif
   context = g_hash_table_lookup (self->priv->contexts, context_id);
 
   if (context)
     {
       /* silently ignore, since multiple traversals of prepare are done
        * to saturate the graph */
-#if ENABLE_MT
       g_mutex_unlock (self->mutex);
-#endif
       return context;
     }
 
   context             = gegl_operation_context_new ();
   context->operation  = self->operation;
   g_hash_table_insert (self->priv->contexts, context_id, context);
-#if ENABLE_MT
   g_mutex_unlock (self->mutex);
-#endif
   return context;
 }
 
@@ -2181,7 +2162,7 @@ gegl_node_new_child (GeglNode    *parent,
 
   name = first_property_name;
   va_start (var_args, first_property_name);
-  gegl_node_set_valist (node, first_property_name, var_args);
+  gegl_node_set_valist (node, name, var_args);
   va_end (var_args);
 
   if (parent)
