@@ -24,6 +24,12 @@
 
 G_BEGIN_DECLS
 
+/**
+ * When a node in a GEGL graph does processing, it needs context such
+ * as inputs. This structure holds this stuff and is passed to the
+ * node during processing. The data in a context is bound to one
+ * specific rendering/processing job/thread.
+ */
 struct _GeglOperationContext
 {
   GeglOperation *operation;
@@ -66,6 +72,8 @@ void            gegl_operation_context_take_object     (GeglOperationContext *co
 void            gegl_operation_context_set_property    (GeglOperationContext *self,
                                                         const gchar          *name,
                                                         const GValue         *value);
+GValue        * gegl_operation_context_get_value       (GeglOperationContext *self,
+                                                        const gchar          *property_name);
 void            gegl_operation_context_get_property    (GeglOperationContext *self,
                                                         const gchar          *name,
                                                         GValue               *value);
