@@ -17,14 +17,13 @@
  *           2005-2008 Øyvind Kolås
  */
 
-#define GEGL_INTERNAL
-
 #include "config.h"
 
 #include <glib-object.h>
 #include <string.h>
 
-#include "gegl-types.h"
+#include "gegl.h"
+#include "gegl-types-internal.h"
 #include "gegl-utils.h"
 #include "gegl-operation.h"
 #include "gegl-operations.h"
@@ -33,13 +32,18 @@
 #include "gegl-operation-context.h"
 #include "buffer/gegl-region.h"
 
+
+static GHashTable *gtype_hash = NULL;
+G_LOCK_DEFINE_STATIC (gtype_hash);
+
+
 static void
 add_operations (GHashTable *hash,
                 GType       parent)
 {
   GType *types;
   guint  count;
-  gint   no;
+  guint  no;
 
   types = g_type_children (parent, &count);
   if (!types)
@@ -52,22 +56,34 @@ add_operations (GHashTable *hash,
         {
           g_hash_table_insert (hash, g_strdup (operation_class->name), (gpointer) types[no]);
         }
+      if (operation_class->compat_name)
+        {
+          g_hash_table_insert (hash, g_strdup (operation_class->compat_name), (gpointer) types[no]);
+        }
       add_operations (hash, types[no]);
     }
   g_free (types);
 }
 
-static GHashTable *gtype_hash = NULL;
 GType
 gegl_operation_gtype_from_name (const gchar *name)
 {
+  GType ret = 0;
+
+  /* FIXME: We only need mutual exclusion when we initialize. Maybe
+   * move initialization to gegl_init()?
+   */
+  G_LOCK (gtype_hash);
   if (!gtype_hash)
     {
       gtype_hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
 
       add_operations (gtype_hash, GEGL_TYPE_OPERATION);
     }
-  return (GType) g_hash_table_lookup (gtype_hash, name);
+  ret = (GType) g_hash_table_lookup (gtype_hash, name);
+  G_UNLOCK (gtype_hash);
+
+  return ret;
 }
 
 static GSList *operations_list = NULL;
@@ -90,7 +106,11 @@ gchar **gegl_list_operations (guint *n_operations_p)
   if (!operations_list)
     {
       gegl_operation_gtype_from_name ("");
+
+      G_LOCK (gtype_hash);
       g_hash_table_foreach (gtype_hash, addop, NULL);
+      G_UNLOCK (gtype_hash);
+
       operations_list = g_slist_sort (operations_list, (GCompareFunc) strcmp);
     }
 
@@ -119,34 +139,50 @@ gchar **gegl_list_operations (guint *n_operations_p)
 void
 gegl_operation_gtype_cleanup (void)
 {
+  G_LOCK (gtype_hash);
   if (gtype_hash)
     {
       g_hash_table_destroy (gtype_hash);
       gtype_hash = NULL;
     }
+  G_UNLOCK (gtype_hash);
 }
 
+/**
+ * gegl_operation_set_need_rect:
+ * @operation:
+ * @context_id:
+ * @input_pad_name:
+ * @region:
+ *
+ * Calculates the need rect for the node feeding into @input_pad_name
+ * for the given graph evaluation that is identified by
+ * @context_id. The need rect of a node mathematically depends on the
+ * ROI that is being evaluated and is the area that evaluation of the
+ * ROI depends on for the node in question.
+ **/
 static void
-gegl_operation_set_source_region (GeglOperation        *operation,
-                                  gpointer              context_id,
-                                  const gchar         *input_pad_name,
-                                  const GeglRectangle *region)
+gegl_operation_set_need_rect (GeglOperation       *operation,
+                              gpointer             context_id,
+                              const gchar         *input_pad_name,
+                              const GeglRectangle *region)
 {
   GeglNode     *child;         /* the node which need rect we are affecting */
   GeglRectangle child_need;    /* the need rect of the child */
+  GeglPad      *output_pad;
 
   g_return_if_fail (GEGL_IS_OPERATION (operation));
   g_return_if_fail (GEGL_IS_NODE (operation->node));
   g_return_if_fail (input_pad_name != NULL);
 
   {
-    GeglPad *pad = gegl_node_get_pad (operation->node, input_pad_name);
-    if (!pad)
+    GeglPad *input_pad = gegl_node_get_pad (operation->node, input_pad_name);
+    if (!input_pad)
       return;
-    pad = gegl_pad_get_internal_connected_to (pad);
-    if (!pad)
+    output_pad = gegl_pad_get_connected_to (input_pad);
+    if (!output_pad)
       return;
-    child = gegl_pad_get_node (pad);
+    child = gegl_pad_get_node (output_pad);
     if (!child)
       return;
   }
@@ -177,9 +213,10 @@ gegl_operation_set_source_region (GeglOperation        *operation,
     gegl_node_set_need_rect (child, context_id, &child_need);
   }
 }
+
 gboolean
-gegl_operation_calc_source_regions (GeglOperation *operation,
-                                    gpointer       context_id)
+gegl_operation_calc_need_rects (GeglOperation *operation,
+                                gpointer       context_id)
 {
   GSList          *input_pads;
   GeglOperationContext *context;
@@ -188,14 +225,16 @@ gegl_operation_calc_source_regions (GeglOperation *operation,
   context = gegl_node_get_context (operation->node, context_id);
   request = context->need_rect;
 
-  /* for each input, get_required_for_output use gegl_operation_set_source_region() */
+  /* For each input, do get_required_for_output() then use
+   * gegl_operation_set_need_rect()
+   */
   for (input_pads = operation->node->input_pads;input_pads;input_pads=input_pads->next)
     {
       const gchar *pad_name = gegl_pad_get_name (input_pads->data);
       GeglRectangle rect;
       rect = gegl_operation_get_required_for_output (operation, pad_name, &request); 
 
-      gegl_operation_set_source_region (operation, context_id, pad_name, &rect);
+      gegl_operation_set_need_rect (operation, context_id, pad_name, &rect);
     }
   return TRUE;
 }
