@@ -128,17 +128,12 @@ static void copy_data (const GeglPathItem *src,
                        GeglPathItem       *dst)
 {
   InstructionInfo *src_info;
-  InstructionInfo *dst_info;
   gint i;
 
   if (!src)
     return;
 
   src_info = lookup_instruction_info(src->type);
-  dst_info = lookup_instruction_info(dst->type);
-
-/*
-  g_assert (src_info->pairs <= dst_info->pairs);*/
 
   dst->type = src->type;  
   for (i=0;i<(src_info->n_items+1)/2;i++)
@@ -202,7 +197,6 @@ gegl_path_list_append_item  (GeglPathList  *head,
   g_assert (res);
     *res = iter;
 
-  tail = iter;
   return head;
 }
 
@@ -302,6 +296,7 @@ static GeglPathList *flatten_curve (GeglMatrix3   matrix,
   gfloat f;
   Point res;
   gchar buf[64]="C";
+  GeglPathItem *item=(void*)buf;
 
   copy_data (&self->d, (void*)buf);
   transform_data (matrix, (void*)buf);
@@ -312,7 +307,7 @@ static GeglPathList *flatten_curve (GeglMatrix3   matrix,
       head = gegl_path_list_append (head, 'L', res.x, res.y);
     }
 
-  res = ((GeglPathItem*)buf)->point[2];
+  res = item->point[2];
   head = gegl_path_list_append (head, 'L', res.x, res.y);
 
   return head;
@@ -323,7 +318,6 @@ gegl_path_list_flatten (GeglMatrix3   matrix,
                         GeglPathList *original)
 {
   GeglPathList *iter;
-  GeglPathList *prev = NULL;
   GeglPathList *self = NULL;
 
   GeglPathList *endp = NULL;
@@ -340,7 +334,6 @@ gegl_path_list_flatten (GeglMatrix3   matrix,
         endp = self;
       while (endp && endp->next)
         endp=endp->next;
-      prev = iter;
     }
   return self;
 }
@@ -390,6 +383,7 @@ GeglPathList * gegl_path_list_append (GeglPathList *head,
   return head;
 }
 
+#if 0
 static void
 path_calc (GeglPathList *path,
            gdouble       pos,
@@ -397,12 +391,9 @@ path_calc (GeglPathList *path,
            gdouble      *yd)
 {
   GeglPathList *iter = path;
-  gfloat traveled_length = 0;
-  gfloat need_to_travel = 0;
+  gfloat traveled = 0, next_pos = 0;
   gfloat x = 0, y = 0;
 
-  gboolean had_move_to = FALSE;
-
   while (iter)
     {
       /*fprintf (stderr, "%c, %i %i\n", iter->d.type, iter->d.point[0].x, iter->d.point[0].y);*/
@@ -411,21 +402,12 @@ path_calc (GeglPathList *path,
           case 'M':
             x = iter->d.point[0].x;
             y = iter->d.point[0].y;
-            need_to_travel = 0;
-            traveled_length = 0;
-            had_move_to = TRUE;
             break;
 
           case 'L':
             {
               Point a,b;
-
-              gfloat spacing;
-              gfloat local_pos;
               gfloat distance;
-              gfloat offset;
-              gfloat leftover;
-
 
               a.x = x;
               a.y = y;
@@ -433,35 +415,22 @@ path_calc (GeglPathList *path,
               b.x = iter->d.point[0].x;
               b.y = iter->d.point[0].y;
 
-              spacing = 0.2;
-
               distance = point_dist (&a, &b);
+              next_pos += distance;
 
-              leftover = need_to_travel - traveled_length;
-              offset = spacing - leftover;
+              if (pos <= next_pos)
+                {
+                  Point spot;
+                  gfloat ratio = (pos - traveled) / (next_pos - traveled);
 
-              local_pos = offset;
+                  lerp (&spot, &a, &b, ratio);
 
-              if (distance > 0)
-                for (;
-                     local_pos <= distance;
-                     local_pos += spacing)
-                  {
-                    Point spot;
-                    gfloat ratio = local_pos / distance;
+                  *xd = spot.x;
+                  *yd = spot.y;
+                  return;
+                }
 
-                    lerp (&spot, &a, &b, ratio);
-
-                    traveled_length += spacing;
-                    if (traveled_length > pos)
-                      {
-                        *xd = spot.x;
-                        *yd = spot.y;
-                        return;
-                      }
-                  }
-
-              need_to_travel += distance;
+              traveled = next_pos;
 
               x = b.x;
               y = b.y;
@@ -477,22 +446,107 @@ path_calc (GeglPathList *path,
       iter=iter->next;
     }
 }
+#endif
+
+static gboolean
+path_calc2 (GeglPathList *path,
+           gdouble       pos,
+           gdouble      *xd,
+           gdouble      *yd,
+           GeglPathList **stop,
+           gdouble      *leftover)
+{
+  GeglPathList *iter = path, *prev = NULL;
+  gfloat traveled = 0.0, next_pos = 0.0;
+
+    
+
+  while (iter && !prev)
+  /* fetch the start point of the path */
+    {
+      /* fprintf (stderr, "%c, %f %f\n", iter->d.type, iter->d.point[0].x, iter->d.point[0].y);*/
+      switch (iter->d.type)
+        {
+          case 'M':
+          case 'L':
+            prev = iter;
+            break;
+          default :
+            break;
+        }
+      iter = iter->next;          
+    }
+
+  while (iter)
+  /* travel along the path */
+    {
+      /* fprintf (stderr, "%c, %f %f\n", iter->d.type, iter->d.point[0].x, iter->d.point[0].y);*/
+      switch (iter->d.type)
+        {
+          case 'M':
+            prev = iter;
+            break;
+
+          case 'L':
+            {
+              Point a,b;
+              gfloat distance;
+
+              a.x = prev->d.point[0].x;
+              a.y = prev->d.point[0].y;
+
+              b.x = iter->d.point[0].x;
+              b.y = iter->d.point[0].y;
+
+              distance = point_dist (&a, &b);
+              next_pos += distance;
+
+              if (pos <= next_pos)
+                {
+                  Point spot;
+                  gfloat ratio = (pos - traveled) / (next_pos - traveled);
+
+                  lerp (&spot, &a, &b, ratio);
+
+                  *xd = spot.x;
+                  *yd = spot.y;
+
+                  *stop = prev;
+                  *leftover += traveled;
+                  return TRUE;
+                }
+
+              traveled = next_pos;
+
+              prev = iter;
+            }
+            break;
+          case 's':
+            break;
+          default:
+            g_warning ("can't compute length for instruction: %c\n", iter->d.type);
+            break;
+        }
+      iter=iter->next;
+    }
+  /*fprintf (stderr, "outside iterator bounds");*/
+  return FALSE;
+}
 
 static void path_calc_values (GeglPathList *path,
                               guint         num_samples,
                               gdouble      *xs,
                               gdouble      *ys)
 {
+  GeglPathList *iter = path;
   gdouble length = path_get_length (path);
+  gfloat spacing = length / (num_samples-1);
+  
+  gfloat traveled = 0, next_pos = 0, next_sample = 0;
+  gfloat x = 0, y = 0;
+  
   gint i=0;
-  gfloat traveled_length = 0;
-  gfloat need_to_travel = 0;
-  gfloat x = 0,y = 0;
-  gboolean had_move_to = FALSE;
-  GeglPathList *iter;
-  gfloat spacing = length / num_samples;
 
-  iter = path;
   while (iter)
     {
       /*fprintf (stderr, "%c, %i %i\n", iter->d.type, iter->d.point[0].x, iter->d.point[0].y);*/
@@ -501,19 +555,11 @@ static void path_calc_values (GeglPathList *path,
           case 'M':
             x = iter->d.point[0].x;
             y = iter->d.point[0].y;
-            need_to_travel = 0;
-            traveled_length = 0;
-            had_move_to = TRUE;
             break;
           case 'L':
             {
               Point a,b;
-
-              gfloat local_pos;
               gfloat distance;
-              gfloat offset;
-              gfloat leftover;
-
 
               a.x = x;
               a.y = y;
@@ -522,40 +568,31 @@ static void path_calc_values (GeglPathList *path,
               b.y = iter->d.point[0].y;
 
               distance = point_dist (&a, &b);
+              next_pos += distance;
 
-              leftover = need_to_travel - traveled_length;
-              offset = spacing - leftover;
+              while (next_sample <= next_pos)
+                {
+                  Point spot;
+                  gfloat ratio = (next_sample - traveled) / (next_pos - traveled);
 
-              local_pos = offset;
+                  lerp (&spot, &a, &b, ratio);
+                  
+                  xs[i]=spot.x;
+                  ys[i]=spot.y;
 
-              if (distance > 0)
-                for (;
-                     local_pos <= distance;
-                     local_pos += spacing)
-                  {
-                    Point spot;
-                    gfloat ratio = local_pos / distance;
-
-                    lerp (&spot, &a, &b, ratio);
-
-                    /*gegl_path_stamp (buffer, clip_rect,
-                      spot.x, spot.y, radius, hardness, color, gopacity);*/
-                    xs[i]=spot.x;
-                    ys[i]=spot.y;
-
-                    traveled_length += spacing;
-                    i++;
-                  }
+                  next_sample += spacing;
+                  i++;
+                }
               if (!iter->next)
                 {
                   xs[num_samples-1]=b.x;
                   ys[num_samples-1]=b.y;
                 }
 
-              need_to_travel += distance;
-
               x = b.x;
               y = b.y;
+
+              traveled = next_pos;
             }
 
             break;
@@ -665,6 +702,10 @@ struct _GeglPathPrivate
   gdouble       length;
   gboolean      length_clean;
 
+  GeglPathList *calc_stop;
+  gdouble       calc_leftover;
+  gboolean      calc_clean;
+
   GeglRectangle dirtied;
   GeglRectangle cached_extent;
   GeglMatrix3   matrix;
@@ -805,6 +846,7 @@ static void ensure_flattened (GeglPath *vector)
     gegl_path_list_destroy (path);
   priv->flat_path_clean = TRUE;
   priv->length_clean = FALSE;
+  priv->calc_clean = FALSE;
 }
 
 static void
@@ -933,6 +975,7 @@ gegl_path_new (void)
   gegl_path_init (self);
   priv->flat_path_clean = FALSE;
   priv->length_clean = FALSE;
+  priv->calc_clean = FALSE;
 
   return self;
 }
@@ -1015,17 +1058,50 @@ void gegl_path_get_bounds (GeglPath *self,
 
 
 
-void
+gboolean
 gegl_path_calc (GeglPath   *self,
                 gdouble     pos,
                 gdouble    *xd,
                 gdouble    *yd)
 {
   GeglPathPrivate *priv = GEGL_PATH_GET_PRIVATE (self);
+  GeglPathList *entry = priv->flat_path;
+  GeglPathList *stop  = NULL;
+  gdouble rel_pos  = 0.0;
+  gdouble leftover = 0.0;
+  gboolean result = FALSE;
+
   if (!self)
-    return;
+    return FALSE;
   ensure_flattened (self);
+#if 0
   path_calc (priv->flat_path, pos, xd, yd);
+#endif
+  if (priv->calc_clean && (pos > priv->calc_leftover))
+    {
+      entry    = priv->calc_stop;
+      leftover = priv->calc_leftover;
+      rel_pos  = pos - leftover;
+    }
+  else
+    {
+      rel_pos = pos;
+    }
+
+  if (path_calc2 (entry,rel_pos,xd,yd,&stop,&leftover))
+    {
+      priv->calc_stop = stop;
+      priv->calc_leftover = leftover;
+      priv->calc_clean = TRUE;
+
+      result = TRUE;
+    }
+  else
+    {
+      priv->calc_clean = FALSE;
+      result = FALSE;
+    }
+  return result;
 }
 
 
@@ -1259,7 +1335,7 @@ void gegl_path_parse_string (GeglPath *vector,
               switch (info->n_items)
                 {
                   case 0:
-                    priv->path = gegl_path_list_append (priv->path, type, x0, y0);
+                    priv->path = gegl_path_list_append (priv->path, type, 0., 0.);
                     /* coordinates are ignored, all of these could have used add3)*/
                     break;
                   case 2:
@@ -1668,8 +1744,6 @@ gegl_path_closest_point (GeglPath *path,
       GeglPathPrivate *priv = GEGL_PATH_GET_PRIVATE (path);
       GeglPathList *iter;
       /* what node was the one before us ? */
-      iter = priv->path;
-      i=0;
 
       for (iter=priv->path,i=0; iter;i++,iter=iter->next)
         {
@@ -1961,9 +2035,9 @@ static void gegl_path_stamp (GeglBuffer *buffer,
                              gdouble     opacity)
 {
   gfloat col[4];
-  static StampStatic s = {FALSE,}; /* XXX: 
-                                      we will ultimately leak the last valid
-                                      cached brush. */
+  StampStatic s = {FALSE,}; /* there should be a cache of stamps,
+                               note that stamps are accessed in multiple threads
+                             */
 
   GeglRectangle temp;
   GeglRectangle roi;
@@ -2036,6 +2110,7 @@ static void gegl_path_stamp (GeglBuffer *buffer,
     }
   }
   gegl_buffer_set_unlocked (buffer, &roi, s.format, s.buf, 0);
+  g_free (s.buf);
 }
 
 
@@ -2060,7 +2135,6 @@ void gegl_path_stroke (GeglBuffer *buffer,
   gfloat traveled_length = 0;
   gfloat need_to_travel = 0;
   gfloat x = 0,y = 0;
-  gboolean had_move_to = FALSE;
   GeglPathList *iter;
   gdouble       xmin, xmax, ymin, ymax;
   GeglRectangle extent;
@@ -2102,7 +2176,6 @@ void gegl_path_stroke (GeglBuffer *buffer,
             y = iter->d.point[0].y;
             need_to_travel = 0;
             traveled_length = 0;
-            had_move_to = TRUE;
             break;
           case 'L':
             {
