@@ -231,7 +231,7 @@ gdouble              gegl_path_closest_point  (GeglPath     *path,
  * Compute the coordinates of the path at the @position (length measured from
  * start of path, not including discontinuities).
  */
-void                 gegl_path_calc           (GeglPath     *path,
+gboolean             gegl_path_calc           (GeglPath     *path,
                                                gdouble       pos,
                                                gdouble      *x,
                                                gdouble      *y);
@@ -423,6 +423,10 @@ typedef struct GeglPathList
   GeglPathItem         d;
 } GeglPathList;
 
+
+/***
+ */
+
 /* appends to path list, if head is NULL a new list is created */
 GeglPathList       * gegl_path_list_append    (GeglPathList *head, ...);
 /* frees up a path list */
