@@ -228,7 +228,7 @@ op_affine_class_init (OpAffineClass *klass)
   op_class->prepare                   = gegl_affine_prepare;
   op_class->no_cache                  = TRUE;
 
-  /*filter_class->process               = gegl_affine_process;*/
+  /*filter_class->process             = gegl_affine_process;*/
 
   klass->create_matrix = NULL;
 
@@ -275,7 +275,6 @@ op_affine_class_init (OpAffineClass *klass)
 static void
 op_affine_init (OpAffine *self)
 {
-  gegl_matrix3_identity (self->matrix);
 }
 
 static void
@@ -341,6 +340,34 @@ gegl_affine_set_property (GObject      *object,
     }
 }
 
+static void
+gegl_affine_create_matrix (OpAffine    *affine,
+                           GeglMatrix3  matrix)
+{
+  gegl_matrix3_identity (matrix);
+
+  if (OP_AFFINE_GET_CLASS (affine))
+    OP_AFFINE_GET_CLASS (affine)->create_matrix (affine, matrix);
+}
+
+static void
+gegl_affine_create_composite_matrix (OpAffine    *affine,
+                                     GeglMatrix3  matrix)
+{
+  gegl_affine_create_matrix (affine, matrix);
+
+  if (affine->origin_x || affine->origin_y)
+    gegl_matrix3_originate (matrix, affine->origin_x, affine->origin_y);
+
+  if (gegl_affine_is_composite_node (affine))
+    {
+      GeglMatrix3 source;
+
+      gegl_affine_get_source_matrix (affine, source);
+      gegl_matrix3_multiply (source, matrix, matrix);
+    }
+}
+
 static void
 gegl_affine_bounding_box (gdouble       *points,
                           gint           num_points,
@@ -439,14 +466,14 @@ gegl_affine_get_source_matrix (OpAffine    *affine,
   source = gegl_connection_get_source_node (connections->data)->operation;
   g_assert (IS_OP_AFFINE (source));
 
-  gegl_matrix3_copy (output, OP_AFFINE (source)->matrix);
+  gegl_affine_create_composite_matrix (OP_AFFINE (source), output);
+  /*gegl_matrix3_copy (output, OP_AFFINE (source)->matrix);*/
 }
 
 static GeglRectangle
 gegl_affine_get_bounding_box (GeglOperation *op)
 {
-  OpAffine      *affine  = (OpAffine *) op;
-  OpAffineClass *klass   = OP_AFFINE_GET_CLASS (affine);
+  OpAffine      *affine  = OP_AFFINE (op);
   GeglMatrix3    matrix;
   GeglRectangle  in_rect = {0,0,0,0},
                  have_rect;
@@ -463,21 +490,8 @@ gegl_affine_get_bounding_box (GeglOperation *op)
   if (gegl_operation_source_get_bounding_box (op, "input"))
     in_rect = *gegl_operation_source_get_bounding_box (op, "input");
 
-  /* invoke child's matrix creation function */
-  g_assert (klass->create_matrix);
-  gegl_matrix3_identity (matrix);
-  klass->create_matrix (op, matrix);
+  gegl_affine_create_composite_matrix (affine, matrix);
 
-  if (affine->origin_x || affine->origin_y)
-    gegl_matrix3_originate (matrix, affine->origin_x, affine->origin_y);
-
-  if (gegl_affine_is_composite_node (affine))
-    {
-      GeglMatrix3 source;
-
-      gegl_affine_get_source_matrix (affine, source);
-      gegl_matrix3_multiply (source, matrix, matrix);
-    }
   if (gegl_affine_is_intermediate_node (affine) ||
       gegl_matrix3_is_identity (matrix))
     {
@@ -502,7 +516,7 @@ gegl_affine_get_bounding_box (GeglOperation *op)
   have_points [7] = in_rect.y + in_rect.height ;
 
   for (i = 0; i < 8; i += 2)
-    gegl_matrix3_transform_point (affine->matrix,
+    gegl_matrix3_transform_point (matrix,
                              have_points + i, have_points + i + 1);
 
   gegl_affine_bounding_box (have_points, 4, &have_rect);
@@ -514,12 +528,11 @@ gegl_affine_detect (GeglOperation *operation,
                     gint           x,
                     gint           y)
 {
-  GeglNode *source_node = gegl_operation_get_source_node (operation, "input");
-
-  OpAffine *affine = (OpAffine *) operation;
-  GeglMatrix3   inverse;
-  gdouble   need_points [2];
-  gint      i;
+  OpAffine    *affine      = OP_AFFINE (operation);
+  GeglNode    *source_node = gegl_operation_get_source_node (operation, "input");
+  GeglMatrix3  inverse;
+  gdouble      need_points [2];
+  gint         i;
 
   if (gegl_affine_is_intermediate_node (affine) ||
       gegl_matrix3_is_identity (inverse))
@@ -530,7 +543,7 @@ gegl_affine_detect (GeglOperation *operation,
   need_points [0] = x;
   need_points [1] = y;
 
-  gegl_matrix3_copy (inverse, affine->matrix);
+  gegl_affine_create_matrix (affine, inverse);
   gegl_matrix3_invert (inverse);
 
   for (i = 0; i < 2; i += 2)
@@ -546,7 +559,7 @@ gegl_affine_get_required_for_output (GeglOperation       *op,
                                      const gchar         *input_pad,
                                      const GeglRectangle *region)
 {
-  OpAffine      *affine = (OpAffine *) op;
+  OpAffine      *affine = OP_AFFINE (op);
   GeglMatrix3    inverse;
   GeglRectangle  requested_rect,
                  need_rect;
@@ -560,7 +573,7 @@ gegl_affine_get_required_for_output (GeglOperation       *op,
   context_rect = sampler->context_rect;
   g_object_unref (sampler);
 
-  gegl_matrix3_copy (inverse, affine->matrix);
+  gegl_affine_create_composite_matrix (affine, inverse);
   gegl_matrix3_invert (inverse);
 
   if (gegl_affine_is_intermediate_node (affine) ||
@@ -581,9 +594,6 @@ gegl_affine_get_required_for_output (GeglOperation       *op,
   need_points [6] = requested_rect.x;
   need_points [7] = requested_rect.y + requested_rect.height ;
 
-  gegl_matrix3_copy (inverse, affine->matrix);
-  gegl_matrix3_invert (inverse);
-
   for (i = 0; i < 8; i += 2)
     gegl_matrix3_transform_point (inverse,
                              need_points + i, need_points + i + 1);
@@ -601,8 +611,8 @@ gegl_affine_get_invalidated_by_change (GeglOperation       *op,
                                        const gchar         *input_pad,
                                        const GeglRectangle *input_region)
 {
-  OpAffine          *affine  = (OpAffine *) op;
-  OpAffineClass     *klass   = OP_AFFINE_GET_CLASS (affine);
+  OpAffine          *affine = OP_AFFINE (op);
+  GeglMatrix3        matrix;
   GeglRectangle      affected_rect;
   GeglRectangle      context_rect;
   GeglSampler       *sampler;
@@ -613,23 +623,22 @@ gegl_affine_get_invalidated_by_change (GeglOperation       *op,
   sampler = op_affine_sampler (OP_AFFINE (op));
   context_rect = sampler->context_rect;
   g_object_unref (sampler);
-  /* invoke child's matrix creation function */
-  g_assert (klass->create_matrix);
-  gegl_matrix3_identity (affine->matrix);
-  klass->create_matrix (op, affine->matrix);
+
+  gegl_affine_create_matrix (affine, matrix);
 
   if (affine->origin_x || affine->origin_y)
-    gegl_matrix3_originate (affine->matrix, affine->origin_x, affine->origin_y);
+    gegl_matrix3_originate (matrix, affine->origin_x, affine->origin_y);
 
   if (gegl_affine_is_composite_node (affine))
     {
       GeglMatrix3 source;
 
       gegl_affine_get_source_matrix (affine, source);
-      gegl_matrix3_multiply (source, affine->matrix, affine->matrix);
+      gegl_matrix3_multiply (source, matrix, matrix);
     }
+
   if (gegl_affine_is_intermediate_node (affine) ||
-      gegl_matrix3_is_identity (affine->matrix))
+      gegl_matrix3_is_identity (matrix))
     {
       return region;
     }
@@ -652,12 +661,11 @@ gegl_affine_get_invalidated_by_change (GeglOperation       *op,
   affected_points [7] = region.y + region.height ;
 
   for (i = 0; i < 8; i += 2)
-    gegl_matrix3_transform_point (affine->matrix,
+    gegl_matrix3_transform_point (matrix,
                              affected_points + i, affected_points + i + 1);
 
   gegl_affine_bounding_box (affected_points, 4, &affected_rect);
   return affected_rect;
-
 }
 
 
@@ -670,7 +678,7 @@ affine_generic (GeglBuffer  *dest,
   GeglBufferIterator *i;
   const GeglRectangle *dest_extent;
   gint                  x, y;
-  gfloat               *dest_buf,
+  gfloat * restrict     dest_buf,
                        *dest_ptr;
   GeglMatrix3           inverse;
   GeglMatrix2           inverse_jacobian;
@@ -861,10 +869,13 @@ gegl_affine_process (GeglOperation        *operation,
 {
   GeglBuffer          *input;
   GeglBuffer          *output;
+  GeglMatrix3          matrix;
   OpAffine            *affine = (OpAffine *) operation;
 
+  gegl_affine_create_composite_matrix (affine, matrix);
+
   if (gegl_affine_is_intermediate_node (affine) ||
-      gegl_matrix3_is_identity (affine->matrix))
+      gegl_matrix3_is_identity (matrix))
     {
       /* passing straight through (like gegl:nop) */
       input  = gegl_operation_context_get_source (context, "input");
@@ -876,8 +887,8 @@ gegl_affine_process (GeglOperation        *operation,
 
       gegl_operation_context_take_object (context, "output", G_OBJECT (input));
     }
-  else if (gegl_affine_matrix3_allow_fast_translate (affine->matrix) ||
-           (gegl_matrix3_is_translate (affine->matrix) &&
+  else if (gegl_affine_matrix3_allow_fast_translate (matrix) ||
+           (gegl_matrix3_is_translate (matrix) &&
             ! strcmp (affine->filter, "nearest")))
     {
       /* doing a buffer shifting trick, (enhanced nop) */
@@ -885,8 +896,8 @@ gegl_affine_process (GeglOperation        *operation,
 
       output = g_object_new (GEGL_TYPE_BUFFER,
                              "source",    input,
-                             "shift-x",   (int)-affine->matrix[0][2],
-                             "shift-y",   (int)-affine->matrix[1][2],
+                             "shift-x",   (int)-matrix[0][2],
+                             "shift-y",   (int)-matrix[1][2],
                              "abyss-width", -1,  /* turn of abyss
                                                     (relying on abyss
                                                     of source) */
@@ -900,7 +911,7 @@ gegl_affine_process (GeglOperation        *operation,
       if (input != NULL)
         g_object_unref (input);
     }
-  else if (gegl_affine_matrix3_allow_fast_reflect_x (affine->matrix))
+  else if (gegl_affine_matrix3_allow_fast_reflect_x (matrix))
     {
       GeglRectangle      src_rect;
       GeglSampler       *sampler;
@@ -926,7 +937,7 @@ gegl_affine_process (GeglOperation        *operation,
       if (input != NULL)
         g_object_unref (input);
     }
-  else if (gegl_affine_matrix3_allow_fast_reflect_y (affine->matrix))
+  else if (gegl_affine_matrix3_allow_fast_reflect_y (matrix))
     {
       GeglRectangle      src_rect;
       GeglSampler       *sampler;
@@ -963,7 +974,7 @@ gegl_affine_process (GeglOperation        *operation,
       sampler = op_affine_sampler (affine);
       g_object_set(sampler, "buffer", input, NULL);
       gegl_sampler_prepare (sampler);
-      affine_generic (output, input, affine->matrix, sampler);
+      affine_generic (output, input, matrix, sampler);
       g_object_unref(sampler->buffer);
       sampler->buffer = NULL;
       g_object_unref (sampler);
